# Lugate
Lugate is a library for building JSON-RPC 2.0 Gateway API just inside of your NGINX configuration file

[![Build Status](https://travis-ci.org/zinovyev/lugate.svg?branch=master)](https://travis-ci.org/zinovyev/lugate)
[![GitHub license](https://img.shields.io/badge/license-MIT-blue.svg)](https://raw.githubusercontent.com/zinovyev/lugate/master/LICENSE)

## About
Lugate is a binding over OpenResty's [ngx\_http\_lua\_module](https://github.com/openresty/lua-nginx-module) module.
This library provides parsing, validating and routing features for JSON-RPC 2.0 protocol.

## Install
Lugate can be installed via the luarocks package manager. Just run:
```bash
luarocks install lugate
```

## Synopsis
```lua
    location / {
          # MIME type determined by default_type:
          default_type 'application/json';

          content_by_lua_block {
              -- Load lugate module
              local Lugate = require "lugate"

              -- Get new lugate instance
              local lugate = Lugate:init({
                routes = {
                  ['v1%.([^%.]+).*'] = '/v1.%1', -- v1.math.subtract -> /v1.math
                  ['v2%.([^%.]+).*'] = '/v2.%1', -- v2.math.addition -> /v2.math
                }
              })

              -- Send multi requst and get multi response
              lugate:run()

              -- Print responses out
              lugate:print_responses()
          }
    }
```

## Lugate proxy call
Lugate brings its own protocol for transferring some additional logic over JSON-RPC 2.0 request to live. It adds
some additional capabilities for routing and caching.
The *params* member of the [request object](http://www.jsonrpc.org/specification#request_object) gets some additional mandatory members:

* **route** - fo the routing note
* **cache** - for the caching lifetime
* **key** - the cache key
* **params** - the regular array of parameter values

After the request is processed by the Lugate module, the **route** and **cache** values are removed from the
*params* member and the **params** value is expanded to fill the whole *params* field.

```json
--> {"jsonrpc": "2.0", "method": "subtract", "params": {"cache":3600, "key": "foobar", "route": "v2.substract", "params": [42, 23]}, "id": 1}
<-- {"jsonrpc": "2.0", "result": -19, "id": 2}

```

## API reference
To get the API reference use this [autogenerated doc](http://zinovyev.github.io/lugate).

## Testing
Use `busted` framework for running tests.

## Running example
The example project uses vagrant provision.
So you need to have vagrant and virtualbox installed on your PC.
Launch project by running this command from the example directory:
```bash
vagrant up
```
Add this line to your local `/etc/hosts` file:
```
192.168.50.47 gateway.lugate.loc service01.lugate.loc service02.lugate.loc service03.lugate.loc
```
